@@ -5,66 +5,73 @@ error_chain! {
 
     errors {
         BytesEmpty {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Incomplete Number Of Bytes")
             display("Incomplete Number Of Bytes At {:?}", pos)
         }
         InvalidByte {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Invalid Byte Found")
             display("Invalid Byte Found At {:?}", pos)
         }
         InvalidIntNoDelimiter {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Invalid Integer Found With No Delimiter")
             display("Invalid Integer Found With No Delimiter At {:?}", pos)
         }
         InvalidIntNegativeZero {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Invalid Integer Found As Negative Zero")
             display("Invalid Integer Found As Negative Zero At {:?}", pos)
         }
         InvalidIntZeroPadding {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Invalid Integer Found With Zero Padding")
             display("Invalid Integer Found With Zero Padding At {:?}", pos)
         }
         InvalidIntParseError {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Invalid Integer Found To Fail Parsing")
             display("Invalid Integer Found To Fail Parsing At {:?}", pos)
         }
         InvalidKeyOrdering {
-            pos: Option<usize>,
+            pos: usize,
             key: Vec<u8>
          } {
             description("Invalid Dictionary Key Ordering Found")
             display("Invalid Dictionary Key Ordering Found At {:?} For Key {:?}", pos, key)
         }
         InvalidKeyDuplicates {
-            pos: Option<usize>,
+            pos: usize,
             key: Vec<u8>
          } {
             description("Invalid Dictionary Duplicate Keys Found")
             display("Invalid Dictionary Key Found At {:?} For Key {:?}", pos, key)
         }
         InvalidLengthNegative {
-            pos: Option<usize>
+            pos: usize
          } {
             description("Invalid Byte Length Found As Negative")
             display("Invalid Byte Length Found As Negative At {:?}", pos)
         }
         InvalidLengthOverflow {
-            pos: Option<usize>
+            pos: usize
          } {
-            description("Invalid Byte Length Found To Overflow Native Size")
-            display("Invalid Byte Length Found To Overflow Native Size At {:?}", pos)
+            description("Invalid Byte Length Found To Overflow Buffer Length")
+            display("Invalid Byte Length Found To Overflow Buffer Length At {:?}", pos)
+        }
+        InvalidRecursionExceeded {
+            pos: usize,
+            max: usize
+        } {
+            description("Invalid Recursion Limit Exceeded")
+            display("Invalid Recursion Limit Exceeded At {:?} For Limit {:?}", pos, max)
         }
     }
 }
